from setuptools import setup, find_packages

setup(
    name='mvproto',
    version='0.5',
    packages=find_packages(),
    description='A secure communication protocol for e2ee.',
    author='Alfisene Keita',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
